SUBROUTINE SUPARAR(YDGEOMETRY,YGFL,YDML_PHY_MF,KULOUT)

!**** *SUPARAR*   - Initialize common parameters used in physics for AROME
!                   and SURFEX

!     Purpose.
!     --------
!           Initialize MODD_PARAMETERS, MODD_CST, MODD_CONF,
!           MODD_RAIN_ICE_DESCR, MODD_RAIN_ICE_PARAM, MODD_BUDGET
!           parameters used in meso_NH Physics and aladin/meso_NH physics
!           interface

!**   Interface.
!     ----------
!        *CALL* *SUPARAR(KULOUT)

!        Explicit arguments :
!        --------------------
!        KULOUT : Logical unit for the output

!        Implicit arguments :
!        --------------------
!        COMMON YOMPHY2

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        Documentation AROME

!     Author.
!     -------
!        R. Zaaboul

!     Modifications.
!     --------------
!        Original : 28-Feb-2006
!     17-Apr-2007 S.Ivatek-S: Cleaning of not used ZGPAR and ZGPAR2
!     11-Jan-2008 Y.Seity: delete NDIAGFR for Surfex output replaced by nshists
!     07-Aug-2009 A.Alias: replace NSHISTS by NSFXHISTS
!     Dec-2010 A.Alias: MLSM added when LMSE is used
!     2011-06: M. Jerczynski - some cleaning to meet norms
!     Fev. 2012: S. Riette, CSEDIM=EULE changed to CSEDIM=SPLI according to MNH
!     Janv. 2013: Y. Bouteloup add initialization of tuning variables for MF scheme
!     T. Wilhelmsson (Sept 2013) Geometry and setup refactoring.
!     2013-11, J. Masek: Correct inclusion of ACRANEB/ACRANEB2 when LMSE.
!     Fev. 2014: Y. Seity, add xcriauti anc xcriautc setup
!     2014-05-27: J.M. Piriou: default values of XCQVR and GQVPLIM.
!     Nov. 2014: Y. Seity, add LFPREC3D for MOCAGE
!     Nov 2016, S. Riette: New ICE3/ICE4 parameters
!     Jan 2021, C.Wittmann: add LDEPSG,RDEPSRED+RDEPGRED
! End Modifications
!-------------------------------------------------------------------------------

USE MODEL_PHYSICS_MF_MOD , ONLY : MODEL_PHYSICS_MF_TYPE
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK, JPHOOK
USE YOM_YGFL  ,ONLY : TYPE_GFLD
USE GEOMETRY_MOD, ONLY : GEOMETRY
USE YOMCT0    ,ONLY : NUNDEFLD, LSFORCS
USE YOMLUN    ,ONLY : NULNAM
USE YOMGPPB   ,ONLY : GPARBUF

!     ------------------------------------------------------------------

IMPLICIT NONE

TYPE(GEOMETRY),    INTENT(IN) :: YDGEOMETRY
TYPE(MODEL_PHYSICS_MF_TYPE),INTENT(INOUT),TARGET :: YDML_PHY_MF
TYPE(TYPE_GFLD)   ,INTENT(INOUT):: YGFL
INTEGER(KIND=JPIM),INTENT(IN) :: KULOUT

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: INDEF

!     ------------------------------------------------------------------

REAL(KIND=JPHOOK) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "posnam.intfb.h"

LOGICAL , POINTER ::  LOTOWNC
LOGICAL , POINTER ::  LAROBU_ENABLE
LOGICAL , POINTER ::  LOLSMC
LOGICAL , POINTER ::  LFPREC3D
LOGICAL            , POINTER ::  LLCRIT
INTEGER(KIND=JPIM) , POINTER ::  NREFROI1
REAL(KIND=JPRB) , POINTER ::  VSQUALL
INTEGER(KIND=JPIM) , POINTER ::  NDIAGWMAX
INTEGER(KIND=JPIM) , POINTER ::  NSWB_MNH
LOGICAL , POINTER ::  LDIAGWMAX
LOGICAL , POINTER :: LKOGAN
LOGICAL , POINTER :: LMODICEDEP
LOGICAL , POINTER :: LICERAD
LOGICAL , POINTER :: LTOTPREC
LOGICAL , POINTER :: LTOTPRECL
INTEGER(KIND=JPIM) , POINTER ::  NPRINTFR
REAL(KIND=JPRB) , POINTER ::  XCQVR
CHARACTER(LEN=4) ::  CMICRO
CHARACTER(LEN=4) ::  CTURB
REAL(KIND=JPRB) , POINTER ::  GQVTOP
REAL(KIND=JPRB) , POINTER ::  GQVPLIM
LOGICAL , POINTER ::  LQVTOP
INTEGER(KIND=JPIM) , POINTER ::  NREFROI2
LOGICAL , POINTER ::  LSQUALL
INTEGER(KIND=JPIM) , POINTER ::  NPTP
REAL(KIND=JPRB) , POINTER :: RADGR,RADSN

#include "namparar.nam.h"
#include "abor1.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUPARAR',0,ZHOOK_HANDLE)
!Associate for variables not in the include namelists nor allocated in the routine
ASSOCIATE(MACPRS=>YDML_PHY_MF%YRPARAR%MACPRS, MACPRR=>YDML_PHY_MF%YRPARAR%MACPRR, &
 & MINPRR=>YDML_PHY_MF%YRPARAR%MINPRR, MINPRS=>YDML_PHY_MF%YRPARAR%MINPRS, &
 & XSW_BANDS=>YDML_PHY_MF%YRPARAR%XSW_BANDS, MALBSCA=>YDML_PHY_MF%YRPARAR%MALBSCA, &
 & MVQS=>YDML_PHY_MF%YRPARAR%MVQS, MINPRG=>YDML_PHY_MF%YRPARAR%MINPRG, &
 & NGPAR=>YDML_PHY_MF%YRPARAR%NGPAR, &
 & NRRI=>YDML_PHY_MF%YRPARAR%NRRI, NRRL=>YDML_PHY_MF%YRPARAR%NRRL, &
 & MSWDIF=>YDML_PHY_MF%YRPARAR%MSWDIF, MALBDIR=>YDML_PHY_MF%YRPARAR%MALBDIR, &
 & MLSM=>YDML_PHY_MF%YRPARAR%MLSM, MVTS=>YDML_PHY_MF%YRPARAR%MVTS, MGZ0H=>YDML_PHY_MF%YRPARAR%MGZ0H, &
 & MCD=>YDML_PHY_MF%YRPARAR%MCD, &
 & MGZ0=>YDML_PHY_MF%YRPARAR%MGZ0, MACPRG=>YDML_PHY_MF%YRPARAR%MACPRG, &
 & MSWDIR=>YDML_PHY_MF%YRPARAR%MSWDIR, MSNOW=>YDML_PHY_MF%YRPARAR%MSNOW, &
 & NDTCHEM=>YDML_PHY_MF%YRPARAR%NDTCHEM, MRAIN=>YDML_PHY_MF%YRPARAR%MRAIN, MVEMIS=>YDML_PHY_MF%YRPARAR%MVEMIS, &
 & NRR=>YDML_PHY_MF%YRPARAR%NRR, &
 & LMSE=>YDML_PHY_MF%YRARPHY%LMSE, LMPA=>YDML_PHY_MF%YRARPHY%LMPA, &
 & YH=>YGFL%YH, &
 & LRAY=>YDML_PHY_MF%YRPHY%LRAY, LRAYFM15=>YDML_PHY_MF%YRPHY%LRAYFM15, LRAYFM=>YDML_PHY_MF%YRPHY%LRAYFM, &
 & YDPARAR=>YDML_PHY_MF%YRPARAR)
!include namelists variables, or variables allocated in the routine
VSQUALL => YDPARAR%VSQUALL
GQVPLIM => YDPARAR%GQVPLIM
LSQUALL => YDPARAR%LSQUALL
LOTOWNC => YDPARAR%LOTOWNC
RADSN => YDPARAR%RADSN
LDIAGWMAX => YDPARAR%LDIAGWMAX
NPRINTFR => YDPARAR%NPRINTFR
LQVTOP => YDPARAR%LQVTOP
NPTP => YDPARAR%NPTP
LKOGAN => YDPARAR%LKOGAN
LMODICEDEP => YDPARAR%LMODICEDEP
LICERAD => YDPARAR%LICERAD
LTOTPREC => YDPARAR%LTOTPREC
LTOTPRECL => YDPARAR%LTOTPRECL
XCQVR => YDPARAR%XCQVR
NREFROI2 => YDPARAR%NREFROI2
NREFROI1 => YDPARAR%NREFROI1
LAROBU_ENABLE => YDPARAR%LAROBU_ENABLE
NSWB_MNH => YDPARAR%NSWB_MNH
GQVTOP => YDPARAR%GQVTOP
LLCRIT => YDPARAR%LLCRIT
LOLSMC => YDPARAR%LOLSMC
NDIAGWMAX => YDPARAR%NDIAGWMAX
RADGR => YDPARAR%RADGR
LFPREC3D => YDPARAR%LFPREC3D

!     ------------------------------------------------------------------

!         1. Set implicit default values for YOMPARAR
!  certaines � mettre en namelist ?
! for microphysics
!default for time splitting in  microphysics
LFPREC3D=.FALSE.
CMICRO='ICE3'
CTURB='TKEL'

IF (YH%LACTIVE) THEN
  NRR=7
  NRRL=2
  NRRI=4
  CMICRO='ICE4'
ELSE
  NRR=6
  NRRL=2
  NRRI=3
ENDIF
NSWB_MNH=6
NPRINTFR=3600*36
! for squall line academic case
LSQUALL=.FALSE.
NREFROI1=1
NREFROI2=1
VSQUALL=0._JPRB
NPTP=1
LDIAGWMAX=.FALSE.
NDIAGWMAX=1
LOLSMC=.FALSE.
LOTOWNC=.FALSE.

LKOGAN=.FALSE.
LMODICEDEP=.FALSE.
LICERAD=.FALSE.
RADGR=0._JPRB
RADSN=0._JPRB

!default option in EDMFm mass-flux scheme:
LLCRIT=.FALSE.
LTOTPREC=.FALSE.
LTOTPRECL=.FALSE.

! default value for chemical time step factor
NDTCHEM=1_JPIM
! default value for Budget
LAROBU_ENABLE=.FALSE.
! Subgrid rain scheme
!
XCQVR = 0.0_JPRB
GQVPLIM=10000._JPRB
GQVTOP=3.75E-6_JPRB
LQVTOP=.FALSE.
!      2.read namelist for parametrisations and consistency checks

CALL POSNAM(NULNAM,'NAMPARAR')
READ (NULNAM,NAMPARAR)

! Work-around for PGI compiler bug
YDPARAR%CMICRO=CMICRO
YDPARAR%CTURB=CTURB

IF (.NOT.LMSE.AND..NOT.LRAYFM.AND..NOT.LRAYFM15) THEN
  NSWB_MNH=1
ENDIF
IF ( CMICRO /= 'ICE3' .AND. CMICRO /= 'ICE4' .AND. CMICRO /= 'LIMA' .AND. CMICRO /= 'NONE') THEN
  CALL ABOR1("AROME Microphysics (CMICRO) must be ICE3, ICE4 or LIMA")
ENDIF
IF ( (CMICRO == 'ICE4') .AND. .NOT.YH%LACTIVE ) THEN
  CALL ABOR1("ICE4 microphysics requires activation of YH in NAMGFL")
ENDIF
IF (LFPREC3D.AND.YGFL%NGFL_EZDIAG < 4) THEN
  CALL ABOR1 ("With LFPREC3D NGFL_EZDIAG should be >= 4 !")
ENDIF
IF ( CTURB /= 'TKEL' .AND. CTURB /= 'NONE') THEN
  CALL ABOR1("AROME turbulence (CTURB) must be TKEL or NONE")
ENDIF

!       3.Initialisation du buffer contenant les variables pseudo-historiques

! initialisation de la dimension du buffer

NGPAR=0
INDEF=NUNDEFLD
MINPRR=INDEF
MINPRS=INDEF
MINPRG=INDEF
MACPRR=INDEF
MACPRS=INDEF
MACPRG=INDEF
MALBDIR=INDEF
MALBSCA=INDEF
MVTS=INDEF
MVEMIS=INDEF
MSWDIR=INDEF
MSWDIF=INDEF
MRAIN=INDEF
MSNOW=INDEF
MVQS=INDEF
MLSM=INDEF
MGZ0=INDEF
MGZ0H=INDEF
MCD=INDEF

! initialisation des pointeurs

IF (LMPA) THEN
  NGPAR=6
  MINPRR=1
  MACPRR=2
  MINPRS=3
  MACPRS=4
  MINPRG=5
  MACPRG=6
  IF (LMSE.OR.LSFORCS) THEN
    NGPAR=NGPAR+4+4*NSWB_MNH
    MALBDIR=7
    MALBSCA=7+NSWB_MNH
    MVTS=MALBSCA+NSWB_MNH
    MVQS=MVTS+1
    MLSM=MVQS+1
    MVEMIS=MLSM+1
    MSWDIR=MVEMIS+1
    MSWDIF=MSWDIR+NSWB_MNH
  ENDIF
ELSE
  IF (LMSE) THEN
    NGPAR=9
    MRAIN=1
    MSNOW=2
    MVTS=3
    MVQS=4
    MGZ0=5
    MGZ0H=6
    MCD=7
    MLSM=8
    MVEMIS=9
    MALBSCA=10
    MALBDIR=MALBSCA+NSWB_MNH
    NGPAR=NGPAR+2*NSWB_MNH
    IF (LRAY.OR.LRAYFM.OR.LRAYFM15) THEN
      MSWDIR=MALBDIR+NSWB_MNH
      MSWDIF=MSWDIR+NSWB_MNH
      NGPAR=NGPAR+2*NSWB_MNH
    ENDIF
  ENDIF
ENDIF

! allocation du buffer
IF (NGPAR /= 0) THEN
  ALLOCATE (GPARBUF (YDGEOMETRY%YRDIM%NPROMA, NGPAR, YDGEOMETRY%YRDIM%NGPBLKS))
ENDIF

!         4. Write in kulout
WRITE(UNIT=KULOUT,FMT='('' COMMON YOMPARAR '')')
WRITE(UNIT=KULOUT,FMT='('' LSQUALL = '',L5,'' LOLSMC = '',L5,'' LOTOWNC = '',L5)')&
 & LSQUALL,LOLSMC,LOTOWNC
WRITE(UNIT=KULOUT,FMT='('' NREFROI1 = '',I3,&
 & '' NREFROI2 = '',I3,'' VSQUALL = '',E13.6)')NREFROI1, NREFROI2,VSQUALL
WRITE(UNIT=KULOUT,FMT='('' NGPAR = '',I3,&
 & '' MINPRR = '',I3,'' MACPRR = '',I3,'' MINPRS = '',I3,&
 & '' MACPRS = '',I3,'' MINPRG = '',I3,'' MACPRG = '',I3)')&
 & NGPAR,MINPRR,MACPRR,MINPRS,MACPRS,MINPRG,MACPRG
WRITE(UNIT=KULOUT,FMT='('' MCD = '',I3,'' MRAIN = '',I3,'' MSNOW = '',I3)') MCD, MRAIN, MSNOW
WRITE(UNIT=KULOUT,FMT='('' MSWDIR = '',I3,&
 & '' MSWDIF = '',I3)')MSWDIR,MSWDIF
WRITE(UNIT=KULOUT,FMT='('' MALBDIR = '',I3,&
& '' MALBSCA = '',I3,'' MVTS = '',I3,'' MVQS = '',I3,'' MVEMIS = '',I3,&
& '', MLSM = '',I3,'' MGZ0 = '',I3,'' MGZ0H = '',I3,'' NSWB_MNH = '',I3)')&
 & MALBDIR,MALBSCA,MVTS,MVQS,MVEMIS,MLSM,MGZ0,MGZ0H,NSWB_MNH
WRITE(UNIT=KULOUT,FMT='('' NPRINTFR = '',I3, '' NPTP = '',I3)')&
 & NPRINTFR,NPTP
WRITE(UNIT=KULOUT,FMT='('' LDIAGWMAX = '',L5,&
 & '' NDIAGWMAX = '',I3,'' NDTCHEM = '',I3)')&
 & LDIAGWMAX, NDIAGWMAX,NDTCHEM
WRITE(UNIT=KULOUT,FMT='('' NRR = '',I3,&
 & '' NRRL = '',I3,'' NRRI = '',I3)')&
 & NRR, NRRL, NRRI
WRITE(UNIT=KULOUT,FMT='('' CTURB  = '',A4,&
 & '' CMICRO  = '',A4,'' LFPREC3D = '',L5)')&
 & CTURB,CMICRO,LFPREC3D

WRITE(UNIT=KULOUT,FMT=*) 'XCQVR=',XCQVR,' GQVPLIM=',GQVPLIM,' GQVTOP=',GQVTOP,' LQVTOP=',LQVTOP
WRITE(UNIT=KULOUT,FMT='('' LLCRIT = '',L5)') LLCRIT
WRITE(UNIT=KULOUT,FMT='('' LTOTPREC = '',L5)') LTOTPREC
WRITE(UNIT=KULOUT,FMT='('' LTOTPRECL = '',L5)') LTOTPRECL
WRITE(UNIT=KULOUT,FMT='('' LKOGAN = '',L5)') LKOGAN
WRITE(UNIT=KULOUT,FMT='('' LMODICEDEP = '',L5)') LMODICEDEP
WRITE(UNIT=KULOUT,FMT='('' RADSN = '',f6.2)') RADSN
WRITE(UNIT=KULOUT,FMT='('' RADGR = '',f6.2)') RADGR
WRITE(UNIT=KULOUT,FMT='('' LICERAD = '',L5)') LICERAD

! -----------------------------------------------------------------------
END ASSOCIATE
IF (LHOOK) CALL DR_HOOK('SUPARAR',1,ZHOOK_HANDLE)
END SUBROUTINE SUPARAR

